// Copyright (c) 2015, Robert Escriva
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of po6 nor the names of its contributors may be used
//       to endorse or promote products derived from this software without
//       specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef po6_io_mmap_h_
#define po6_io_mmap_h_

// POSIX
#include <sys/mman.h>

// po6
#include <po6/error.h>
#include <po6/noncopyable.h>

namespace po6
{
namespace io
{

class mmap
{
    public:
        mmap(void* addr, size_t length,
             int prot, int flags,
             int fd, off_t offset);
        ~mmap() throw () { close(); }

    public:
        void* get() const { return m_base; }
        size_t size() const { return m_length; }
        bool valid() const { return m_base != NULL; }
        int error() const { return m_error; }
        void close();

    private:
        void* m_base;
        size_t m_length;
        int m_error;

    private:
        PO6_NONCOPYABLE(mmap);
};

inline
mmap :: mmap(void* addr, size_t length,
             int prot, int flags,
             int fd, off_t offset)
    : m_base(NULL)
    , m_length(length)
    , m_error(0)
{
    m_base = ::mmap(addr, length, prot, flags, fd, offset);

    if (m_base == MAP_FAILED)
    {
        m_base = NULL;
        m_error = errno;
    }
}

inline void
mmap :: close()
{
    if (m_base)
    {
        munmap(m_base, m_length);
    }
}

} // namespace io
} // namespace po6

#endif // po6_io_mmap_h_
